@@ -17,107 +17,19 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: Base.py,v 1.18 1999/07/30 12:32:11 gobry Exp $
+# $Id: Base.py,v 1.26 1999/08/27 12:37:18 gobry Exp $
 
 from string import *
-import copy, re
+import re
 import Pyblio.Help
-from Pyblio.Fields import *
 from types import *
 
-Entries = {}
-Types   = {}
+from Pyblio import Config
 
-def getentry (entry):
-	""" Returns an entry description given its name """
-	l = lower (entry)
-	
-	if Entries.has_key (l):
-		return Entries [l]
-	
-	return EntryDescription (entry)
-
-def gettype (entry, field):
-	""" Return a field type given its name and the entry it belongs to """
-	
-	if entry and entry.has_key (field):
-		return entry [field].type
-	
-	if Types.has_key (field):
-		return Types [field].type
-	
-	return TypeText
-
-	
-# ----- Boolean test -----
-
-class Bool:
-	""" Base class for boolean tests """
-
-	def __init__ (self):
-		self.neg = 0
-		
-	def __or__ (self, other):
-		return OrConnect (self, other)
-
-	def __and__ (self, other):
-		return AndConnect (self, other)
-
-	def __neg__ (self):
-		self.neg = not self.neg
-		return self
-
-class Tester (Bool):
-
-	def __init__ (self, field, value):
-		Bool.__init__ (self)
-		self.__test = re.compile (value, re.IGNORECASE)
-
-		self.field = lower (field)
-		self.value = value
-
-	def match (self, entry):
-		if self.neg:
-			# Negative test
-			if entry.has_key (self.field):
-				field = entry [self.field]
-				return field.match (self.__test) == None
-			else:
-				return 1
-		else:
-			if entry.has_key (self.field):
-				field = entry [self.field]
-				return field.match (self.__test) != None
-			else:
-				return 0
-			
-class Connecter (Bool):
-	def __init__ (self, left, right):
-		Bool.__init__ (self)
-		self.left = left
-		self.right = right
+""" This Module contains the base classes one might want to inherit
+from in order to provide a new database format """
 
 
-class OrConnect (Connecter):
-	def match (self, entry):
-		ret = (self.left.match (entry) or
-		       self.right.match (entry)) 
-		if self.neg:
-			ret = not ret
-
-		return ret
-
-class AndConnect (Connecter):
-	def match (self, entry):
-		ret = (self.left.match (entry) and
-			self.right.match (entry))  
-		if self.neg:
-			ret = not ret
-
-		return ret
-
-
-		
 		
 def format (string, width, first, next):
 	"Format a string on a given width"
@@ -164,67 +76,143 @@ class Key:
 			self.base = key.base
 			self.key  = key.key
 		else:
-			self.base = base
 			self.key  = key
+			if type (base) is StringType:
+				self.base = base
+			else:
+				self.base = base.key
 		return
 
 	def __repr__ (self):
+		return "Key (%s, %s)" % (`self.base`, `self.key`)
+	
+	def __str__ (self):
 		if self.base:
-			return `self.base.key + ' - ' + self.key`
+			return str (self.base) + ' - ' + str (self.key)
 		else:
-			return `self.key`
+			return str (self.key)
 		
 	def __cmp__ (self, other):
-		r = cmp (self.base, other.base)
+		try:
+			r = cmp (self.base, other.base)
+		except AttributeError:
+			return 1
+		
 		if r: return r
 		
 		return cmp (self.key, other.key)
 
 	def __hash__ (self):
-		return hash (self.key + '\0' + self.base.key)
+		return hash (self.key + '\0' + self.base)
 
 
 class Entry:
+	"""
+	A database entry. It behaves like a dictionnary, which
+	returns an instance of Description for each key. For example,
+	entry ['author'] is expected to return a Types.AuthorGroup
+	instance.
 
-	id = 'VirtualEntry'
+	Each entry class must define an unique ID, which is used
+	during conversions.
+
+	The entry.name is the name of the entry inside its database,
+	whereas the entry.key is an instance of Key, and has to be
+	unique over the whole application.
+
+	The entry.type is an instance of Types.EntryDescription. It
+	links the field names with their type.
+	"""
 	
-	def __init__ (self, key, name, type, dict):
+	id = 'VirtualEntry'
+
+	def __init__ (self, key, name, type = None, dict = None):
 		self.name = name
 		self.type = type
-		self.__dict = dict
+		self.__dict = dict or {}
 		self.key    = key
-
 		
-	def keys (self):
-		return self.__dict.keys ()
+		self.crossref = None
+		return
 
+	def keys (self):
+		""" returns all the keys for this entry (counting those who belong to
+		crossreferences) """
+		perso = self.__dict.keys ()
+		if self.crossref:
+			for k in self.crossref.keys ():
+				if not self.is_personal (k):
+					perso.append (k)
+
+		return perso
+
+			
+	def personal_keys (self):
+		""" Returns the keys that are actually defined by the entry, and not by its
+		crossreferences """
+		return self.__dict.keys ()
+	
+	def is_personal (self, key):
+		""" indicates if a given field is defined by the entry or one of its
+		crossreferences """
+		if self.__dict.has_key (key):
+			return 1
+		return 0
+	
 	def has_key (self, key):
-		return self.__dict.has_key (key)
+		if self.__dict.has_key (key): return 1
+		
+		if self.crossref:
+			if self.crossref.has_key (key): return 1
+			
+		return 0
 
 	def text (self, key):
 		""" return text with indication of convertion loss """
-		return self.__dict [key], 0
+		
+		if self.is_personal (key):
+			return self.__dict [key], 0
 
-	def native (self, key):
+		if self.crossref:
+			if self.crossref.has_key (key): return self.crossref.text (key)
+			
+		raise KeyError, "entry has no key `%s'" % key
+		
+	def get_native (self, key):
 		""" returns the field in its native form """
+		
 		return self [key]
+
+	def set_native (self, key, value):
+		""" sets the field in its native form """
+		
+		self [key] = value
+		return
 	
 	def __getitem__ (self, key):
-		""" return raw text of a field """
+		""" return text representation of a field """
+		
 		return self.text (key) [0]
 	
 	def foreach (self, function, argument = None):
 		""" To provide compatibility with ref """
+		
 		function (self, argument)
+		return
+
 	
 	def __setitem__ (self, name, value):
 		self.__dict [name] = value
+		return
 
+	
 	def __delitem__ (self, name):
 		del self.__dict [name]
+		return
 
+	
 	def __add__ (self, other):
-		"Merges two entries"
+		""" Merges two entries, key by key """
 
 		ret = Entry (self.key, self.name, self.type, {})
 
@@ -268,8 +256,9 @@ class Entry:
 			try:
 				dico.remove (lcname)
 			except ValueError:
-				raise ValueError, ("error: field `%s' appears more than once " +
-						   "in the definition of `%s'") % (name, tp)
+				raise ValueError,\
+				      "error: field `%s' appears more than once in " +\
+				      "the definition of `%s'" % (name, tp)
 			
 		for f in dico:
 			text = text + '  '
@@ -280,72 +269,140 @@ class Entry:
 		return text
 	
 		
-# Une base de references bibliographiques, comme un dictionnaire
-# avec des extensions...
-
 class DataBase:
+	"""
+	This class represents a full bibliographic database.
+	It also looks like a dictionnary, each key being an instance
+	of class Key.
+
+	The foreach() method should provide an efficient way of
+	looping over all the elements, if one exists. 
+	"""
+
+	# a default database provides no editing facilities
+	# (as it cannot be saved)
+	properties = {}
+
 	id    = 'VirtualDB'
 	__count = 0
+	__keyid = 0
+	
 	
 	def __init__ (self, basename):
 		self.name = basename
 
 		self.key  = 'base-' + str (DataBase.__count)
 		DataBase.__count = DataBase.__count + 1
+
+		self.__dict = {}
 		return
 	
+	def has_property (self, prop):
+		""" indicates if the database has a given property """
 		
+		if self.properties.has_key (prop):
+			return self.properties [prop]
+		
+		return 0
+	
+	def new_entry (self):
+		""" returns a newly created entry """
+
+		name = None
+		key  = None
+		
+		# generate a new key
+		while not name and not self.has_key (key):
+			name = "entry-%d" % self.__keyid
+			key = Key (self, name)
+			
+			self.__keyid = self.__keyid + 1
+
+		# get the first type
+		type = Config.get ("base/defaulttype").data
+
+		# return the new entry
+		return Entry (key, name, type, {})
+
+
 	def keys (self):
-		""
-		raise NameError, "not inherited"
+		""" Returns a list of all the keys available for the database """
+		return self.__dict.keys ()
 
-	def has_key (self, field, key):
-		""
-		raise NameError, "not inherited"
+	def has_key (self, key):
+		""" Tests for a given key """
+		return self.__dict.has_key (key)
 
-	def __getitem__ (self, name):
-		""
-		raise NameError, "not inherited"
+	def __getitem__ (self, key):
+		""" Returns the Entry object associated with the key """
+		return self.__dict [key]
 
-	def __setitem__ (self, name, value):
-		""
-		raise NameError, "not inherited"
+	def __setitem__ (self, key, value):
+		""" Sets a key Entry """
+		
+		self.__dict [key] = value
+		return
 
-	def __delitem__ (self, name):
-		""
-		raise NameError, "not inherited"
-
-	def __len__ (self, name):
-		""
+	def __delitem__ (self, key):
+		""" Removes an Entry from the database, by its key """
+		del self.__dict [key]
+		return
+	
+	def __len__ (self):
+		""" Number of entries in the database """
 		return len (self.keys ())
 
 	def __repr__ (self):
-		""
 		return "<generic bibliographic database (" + `len (self)` + \
 		       " entries)>"
 		
-	# Specifique a la base biblio
-	def where (self, key):
-		"Chercher une entree selon des criteres"
-		raise NameError, "not inherited"
 
+	def where (self, test):
+		""" Returns a subset of the entries according to a
+		criterion """
+		keys = []
+		for e in self.keys ():
+			if test.match (self [e]):
+				keys.append (e)
+
+		refs = Reference ()
+		
+		if len (keys):
+			refs.add (self, keys)
+
+		refs.properties = self.properties
+		return refs
+	
 	def foreach (self, function, args = None):
-		"Parcourt de toutes les entrees"
-		raise NameError, "not inherited"
+		""" Run function on every entry. """
+		
+		for e in self.keys ():
+			function (self [e], args)
+		return
 
 	def update (self):
-		"Remise a jour des donnes"
-		raise NameError, "not inherited"
+		""" Updates the Entries stored in the database """
+		
+		raise IOError, "no update method defined"
+		return
 
 
-	def sort (self, sortkey):
-		""" sort entries according to a given key """
+	def sort (self, sortkey = None):
+
+		""" sort entries according to a given key.  If the
+		sortkey is None, sort according to the entry key """
+		
 		# collect key values
 		dict = {}
 		none = []
 		
 		def collect (entry, arg):
 			sortkey, dict, none = arg
+
+			if sortkey is None:
+				# sort by key
+				dict [entry.key] = entry.key
+				return
 			
 			if entry.has_key (sortkey):
 				dict [entry.key] = entry [sortkey]
@@ -363,7 +420,9 @@ class DataBase:
 		
 		r = Reference ()
 		r.add (self, none + keys)
-		
+
+		# simple search, copy the properties
+		r.properties = self.properties
 		return r
 
 
@@ -394,49 +453,34 @@ class Reference (DataBase):
 
 	__count = 0
 	id = 'Reference'
-		
+
+
 	def __init__ (self):
 		self.__refs  = []
 		self.__bases = {}
 		
 		self.key  = 'ref-' + str (Reference.__count)
 		Reference.__count = Reference.__count + 1
+
 		return
 	
 
 	def add (self, base, liste = None):
-		"Ajouter des references liees a une base"
-
-		target = None
-		if self.__bases.has_key (base.key):
-			target = self.__bases [base.key] [1]
-
-		if target is None:
-			# add a new database
-			local = ( base, [] )
-			target = local [1]
-
-			self.__bases [base.key] = local
-			self.__refs.append (local)
-
-		if liste is None: return
+		""" Ajouter des references liees a une base """
 
 		if base.id == 'Reference':
-			if not (type (liste) is ListType):
-				target.append (liste)
-				return
-		
-			for i in liste:
-				target.append (i)
-				
+			for b in base.bases ():
+				self.__bases [b.key] = b
 		else:
-			if not (type (liste) is ListType):
-				target.append (liste.key)
-				return
+			self.__bases [base.key] = base
 		
-			for i in liste:
-				target.append (i.key)
-				
+		if liste is None:
+			liste = base.keys ()
+
+		if type (liste) is ListType:
+			self.__refs = self.__refs + liste
+		else:
+			self.__refs.append (liste)
 		return
 
 
@@ -448,81 +492,66 @@ class Reference (DataBase):
 		""" Returns the databases available in the Reference
 		object """
 		
-		bases = []
-		for b in self.__refs:
-			bases.append (b [0])
-			
-		return bases
-
+		return self.__bases.values ()
+	
+	def base (self, entry):
+		""" Returns the database the entry actually comes from """
+		return self.__bases [entry.key.base]
+	
 	def refs (self, base):
 		""" Returns the references available in a given
 		database """
 		
 		if self.__bases.has_key (base.key):
-			return map (lambda x, base = base: Key (base, x),
-				    self.__bases [base.key] [1])
+			return filter (lambda x, base = base:
+				       x.base == base.key,
+				       self.__refs)
 		
 		return None
 
 	def base_foreach (self, base, function, arg = None):
 		""" Foreach entry in a base """
 		
-		liste = self.refs (base)
-
-		if liste is None: return
-		
-		if len (liste) > 0:
-			# a subset of the database
-			for e in liste:
-				entry = base [Key (base, e)]
-				function (entry, arg)
-		else:
-			# whole database
-			base.foreach (function, arg)
-
-			
+		for k in self.refs (base):
+			entry = base [k]
+			function (entry, arg)
+		return
+	
 	def foreach (self, function, arg = None):
 		"Foreach entry"
 
-		for b in self.bases ():
-			self.base_foreach (b, function, arg)
-
+		for k in self.keys ():
+			function (self [k], arg)
+		return
+		
 
 	def __len__ (self):
-		total = 0
-		for b in self.bases ():
-			refs = self.refs (b)
-			if refs == []:
-				total = total + len (b)
-			else:
-				total = total + len (refs)
-
-		return total
+		return len (self.keys ())
 
 
 	def keys (self):
-		keys = []
-
-		def appender (entry, keys):
-			keys.append (entry.key)
-			return
-		
-		self.foreach (appender, keys)
-		
-		return keys
-	
+		return self.__refs
 					  
 	def has_key (self, key):
-		return key.base.has_key (key)
-
+		return self.__refs.count (key)
+	
 	def __getitem__ (self, key):
-		return key.base [key]
+		return self.__bases [key.base] [key]
 
 	def __delitem__ (self, key):
-		del key.base [key]
+		del self.__bases [key.base] [key]
 
 	def __setitem__ (self, key, value):
-		key.base [key] = value
+		self.__bases [key.base] [key] = value
+
+		# eventually add it to the current list
+		if not self.has_key (key):
+			# check the database
+			if not self.__bases.has_key (key.base):
+				raise KeyError, "database `%s' is unknown in this Reference" \
+				      % key.base
+			
+			self.__refs.append (key)
 		return
 	
 	def __add__ (self, other):
@@ -537,59 +566,34 @@ class Reference (DataBase):
 
 		return r
 
+	def new_entry (self):
+		if self.has_property ('add'):
+			return self.bases () [0].new_entry ()
+
+		return DataBase.new_entry (self)
+		
+	def update (self):
+		# if we have any edition property
+		if self.has_property ('edit') or self.has_property ('add') or\
+		   self.has_property ('remove'):
+			
+			for b in self.bases ():
+				b.update ()
+			return
+		
+		# else, raise an error
+		raise IOError, "can't update a database of type Reference"
+	
 
 	def where (self, test):
 		r = Reference ()
 
-		for b in self.bases ():
-			list = self.refs (b)
+		for k in self.keys ():
+			entry = self [k]
 			
-			if list is None: continue
-			
-			# Si on doit tester toute une base, on
-			# utilise la commande appropriee
-			if list == []:
-				r = r + b.where (test)
-			else:
-				sublist = []
-				# Sinon on test cas par cas
-				for e in list:
-					key = Key (b, e)
-					if test.match (b [key]):
-						sublist.append (key)
-				
-				# add to the reference
-				if len (sublist) > 0:
-					r.add (b, sublist)
+			if test.match (self [k]):
+				r.add (self.base (entry), k)
 
+		r.properties = self.properties
 		return r
 
-
-	def sort (self, sortkey):
-		""" sort entries according to a given key """
-		
-		# collect key values
-		dict = {}
-		none = []
-		
-		def collect (entry, arg):
-			sortkey, dict, none = arg
-			
-			if entry.has_key (sortkey):
-				dict [entry.key] = entry [sortkey]
-			else:
-				none.append (entry.key)
-			return
-
-		self.foreach (collect, (sortkey, dict, none))
-		
-		keys = dict.keys ()
-		def sort_method (a,b, dict = dict):
-			return cmp (dict [a], dict [b])
-		
-		keys.sort (sort_method)
-		
-		r = Reference ()
-		r.add (self, none + keys)
-		
-		return r
