@@ -160,6 +160,18 @@ class Database (GtkCList):
         Utils.set_cursor (self, 'normal')
 
         return
+
+    def sort_by_key (self):
+        if self.data is None: return
+
+        Utils.set_cursor (self, 'clock')
+
+        keys = self.data.sort (None)
+        
+        self.set (keys)
+
+        Utils.set_cursor (self, 'normal')
+        return
         
     def remove (self, key):
         if self.data is None: return
