@@ -17,11 +17,13 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: Pybliographic.py,v 1.15 1999/08/26 11:16:04 gobry Exp $
+# $Id: Pybliographic.py,v 1.16 1999/08/27 12:37:42 gobry Exp $
 
-import string, os
+import string, os, urlparse
 
 from gnome.ui import *
+from gnome import config
+
 from gtk import *
 import GtkExtra, GDK, GTK
 
@@ -107,8 +109,22 @@ class URLFileSelection (GtkFileSelection):
         
         if self.ret [-1] == '/':
             if self.url:
-                self.ret = self.url.get_text ()
-                if self.ret == "": self.ret = None
+                ret = self.url.get_text ()
+                
+                if ret == "":
+                    self.ret = None
+                    return
+                
+                # construct a nice URL
+                if string.lower (ret [0:5]) != 'http:' and \
+                   string.lower (ret [0:4]) != 'ftp:':
+                    
+                    if ret [0:2] != '//':
+                        ret = '//' + ret
+                        
+                    ret = 'http:' + ret
+
+                self.ret = ret
             else:
                 self.ret = None
         
@@ -145,6 +161,7 @@ class Pybliographic (GnomeApp):
             UIINFO_ITEM_STOCK ('Save As...', None, self.file_save_as,
                                STOCK_MENU_SAVE_AS),
             UIINFO_SEPARATOR,
+            UIINFO_SEPARATOR,
             UIINFO_ITEM_STOCK ('Quit', None, self.file_exit, STOCK_MENU_QUIT),
             ]
         
@@ -166,6 +183,8 @@ class Pybliographic (GnomeApp):
             UIINFO_SEPARATOR,
             UIINFO_ITEM_STOCK('Search...', None, self.edit_search,
                               STOCK_MENU_SEARCH),
+            UIINFO_SEPARATOR,
+            UIINFO_ITEM ('Sort by Key', None, self.sort_by_key)
             ]
 
         config_menu = [
@@ -196,6 +215,7 @@ class Pybliographic (GnomeApp):
         self.create_menus (menus)
         self.create_toolbar (toolbar)
 
+        
         # Put information in Paned windows
         paned = GtkVPaned ()
         paned.show ()
@@ -250,6 +270,40 @@ class Pybliographic (GnomeApp):
         self.modified  = 0
 
         self.search = None
+        self.opened_files = []
+        
+        files = list (config.get_vector ("Pybliographer/Base/History="))
+        for f in files:
+            if f == '': continue
+            
+            self.add_opened_file (f)
+            
+        return
+
+    # --------------------------------------------------
+    def add_opened_file (self, file):
+        
+        if self.opened_files.count (file): return
+        
+        def opener (widget, self = self, file = file):
+            self.open (file)
+
+        if len (file) > 20:
+            s = os.path.split (file)
+            shortfile = '...' + s [0] [-10:] + '/' + s [1]
+        else:
+            shortfile = file
+            
+        self.insert_menus ("File/<Separator>", [
+            UIINFO_ITEM (shortfile, None, opener)])
+
+        self.opened_files.append (file)
+        return
+    
+    # --------------------------------------------------
+        
+    def sort_by_key (self, *arg):
+        self.database.sort_by_key ()
         return
     
     # --------------------------------------------------
@@ -260,7 +314,7 @@ class Pybliographic (GnomeApp):
     # --------------------------------------------------
 
     def file_new (self, *arg):
-        win = URLFileSelection ("Save", FALSE, TRUE, FALSE)
+        win = URLFileSelection ("New file", FALSE, TRUE, FALSE)
         win.set_filename (self.directory)
 
         file = win.run ()
@@ -271,6 +325,17 @@ class Pybliographic (GnomeApp):
 
 
     def new (self, file, type):
+        def callback (arg, self = self):
+            if arg == 0: self.modified = 0
+            return
+
+        if self.modified:
+            d = GnomeQuestionDialog ("Database is modified.\n" +
+                                     "Discard changes ?", callback)
+            d.run_and_close ()
+            
+        if self.modified: return
+
         try:
             self.database.set (Open.bibnew (file, type))
             
@@ -283,6 +348,8 @@ class Pybliographic (GnomeApp):
         
         self.status.pop ()
         self.status.push ("Database `%s' [%s]" % (self.filename, self.type))
+
+        self.add_opened_file (file)
         return
     
     # --------------------------------------------------
@@ -290,6 +357,17 @@ class Pybliographic (GnomeApp):
     def open (self, file, how = None):
         """ open a database """
         
+        def callback (arg, self = self):
+            if arg == 0: self.modified = 0
+            return
+
+        if self.modified:
+            d = GnomeQuestionDialog ("Database is modified.\n" +
+                                     "Discard changes ?", callback)
+            d.run_and_close ()
+            
+        if self.modified: return
+
         Utils.set_cursor (self, 'clock')
 
         try:
@@ -310,6 +388,9 @@ class Pybliographic (GnomeApp):
         self.status.push ("Database `%s' [%s, %d entries]"
                    % (self.filename, self.type, len (self.database.data.keys ())))
 
+        self.add_opened_file (file)
+        return
+
 
     def file_open (self, *arg):
         """ callback for the Open button """
@@ -336,7 +417,12 @@ class Pybliographic (GnomeApp):
                                      "Quit without saving ?", callback)
             d.run_and_close ()
             
-        if not self.modified: self.quit ()
+        if not self.modified:
+            # save the 10 last files
+            config.set_vector ("Pybliographer/Base/History", self.opened_files [-10:])
+            config.sync ()
+            
+            self.quit ()
         return
     
 
