@@ -17,12 +17,12 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: Search.py,v 1.6 1999/08/26 08:07:41 gobry Exp $
+# $Id: Search.py,v 1.7 1999/08/31 09:49:43 gobry Exp $
 
 from gnome.ui import *
 from gtk import *
 
-import string, re, sys, traceback
+import string, re, sys, traceback, copy
 
 from Pyblio import Types, Search, Config
 
@@ -51,6 +51,7 @@ class ItemStorage (GtkTreeItem):
         self.parent = None
         self.children = []
         self.tree     = None
+
         return
 
     def add (self, new):
@@ -86,7 +87,7 @@ class ItemStorage (GtkTreeItem):
         if not self.tree: return
 
         # remove the children also
-        for child in self.children:
+        for child in copy.copy (self.children):
             # recursively proceed
             child.remove_subtree ()
             
