@@ -17,7 +17,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: TextUI.py,v 1.14 1999/07/30 12:32:13 gobry Exp $
+# $Id: TextUI.py,v 1.17 1999/08/06 12:49:11 gobry Exp $
 
 # --------------------------------------------------
 # Basical `high level' functions
@@ -25,14 +25,14 @@
 
 from types import *
 
-import os, sys, traceback, tempfile
+import os, sys, traceback, tempfile, string
 
-import Pyblio.Base, Pyblio.Help
+from Pyblio import Base, Help, Search, Types
 from Pyblio.Open import bibopen, bibwrite
 
 # ----- Create elementary Test -----
 
-Pyblio.Help.register ('searching', """
+Help.register ('searching', """
 The general syntax of a search is :
 
  dbase.where ( constraints )
@@ -52,16 +52,82 @@ dbase.where (has ('author', 'me') & - has ('title', 'failed'))
 A search returns references on the entries that were found. It is then
 possible to search these references to restrain the result.
 
-See also: references
+For simple searches, you can use the function `search'
+
+See also: search, references
 """)
 
 def has (field, value):
-	return Pyblio.Base.Tester (field, value)
+	return Search.Tester (field, value)
+
+
+Help.register ('search',"""
+Usage: search (database, request)
+
+request is a string like:
+
+	author = weig & title = time
+""")
+
+
+def _split_req (req):
+	test = None
+	
+	list = map (string.strip, string.split (req, '|'))
+	if len (list) > 1:
+		for e in list:
+			if test == None:
+				test = _split_req (e)
+			else:
+				test = test | _split_req (e)
+		return test
+	
+	list = map (string.strip, string.split (req, '&'))
+	if len (list) > 1:
+		for e in list:
+			if test == None:
+				test = _split_req (e)
+			else:
+				test = test & _split_req (e)
+		return test
+
+	list = map (string.strip, string.split (req, '!='))
+	if len (list) > 2:
+		raise SyntaxError, 'invalid test'
+
+	if len (list) == 2:
+		return - Search.Tester (string.lower (string.strip (list [0])),
+				      string.strip (list [1]))
+
+
+	list = map (string.strip, string.split (req, '='))
+	if len (list) > 2:
+		raise SyntaxError, 'invalid test'
+
+	if len (list) == 2:
+		return Search.Tester (string.lower (string.strip (list [0])),
+				      string.strip (list [1]))
+
+	for e in Types.SearchedFields:
+		if test == None:
+			test = Search.Tester (string.lower (e),
+					      string.strip (list [0]))
+		else:
+			test = test | Search.Tester (string.lower (e),
+						   string.strip (list [0]))
+
+	return test
+
+
+def search (base, req):
+	t = _split_req (req)
+	return base.where (t)
+
 
 
 # ----- Generic reference holder -----
 
-Pyblio.Help.register ('ref',"""
+Help.register ('ref',"""
 Syntax : reference = ref (base, [entries])
 
 This function returns a reference object (like .where () method) on
@@ -70,7 +136,7 @@ names).
 """)
 
 def ref (base, list = None):
-	ref = Pyblio.Base.Reference ()
+	ref = Base.Reference ()
 
 	if list == None:
 		ref.add (base)
@@ -98,10 +164,10 @@ def pager_handle (inpager):
 	return pager
 
 
-Pyblio.Help.register ('more', """
-Syntax: more (references, [nopager])
+Help.register ('more', """
+Syntax: more (references, [pager])
 
-Display references in BibTeX format. If nopager is 1, the entries are
+Display references in BibTeX format. If nopager is 0, the entries are
 sent to stdout instead of a pager.
 """)
 
@@ -115,8 +181,8 @@ def more (refs, inpager = 1):
 		print "warning: broken pipe"
 		
 
-Pyblio.Help.register ('ls', """
-Syntax: ls (references, [nopager])
+Help.register ('ls', """
+Syntax: ls (references, [pager])
 
 Display only title/author and identifier for each entry.
 
