@@ -18,11 +18,11 @@
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
- $Id: author.c,v 1.5 1999/08/05 09:43:23 gobry Exp $
+ $Id: author.c,v 1.6 1999/08/27 12:36:23 gobry Exp $
 */
 
 #ifdef HAVE_CONFIG_H
-#include "pyblioconf.h"
+#include "config.h"
 #endif
 
 #include <string.h>
@@ -543,10 +543,15 @@ bibtex_author_parse (BibtexStruct * s,
 	list = list->next;
 
 	if (g_strcasecmp (text, "and") == 0) {
-	    extract_author (authors, aut_elem);
-
-	    g_list_free (aut_elem);
-	    aut_elem = NULL;
+	    if (aut_elem == NULL) {
+		bibtex_warning ("double `and' in author field");
+	    }
+	    else {
+		extract_author (authors, aut_elem);
+		
+		g_list_free (aut_elem);
+		aut_elem = NULL;
+	    }
 	}
 	else {
 	    aut_elem = g_list_append (aut_elem, group);
@@ -554,9 +559,13 @@ bibtex_author_parse (BibtexStruct * s,
     }
 
     /* Extract last author */
-    extract_author (authors, aut_elem);
-    
-    g_list_free (aut_elem);
+    if (aut_elem == NULL) {
+	bibtex_warning ("`and' at end of author field");
+    }
+    else {
+	extract_author (authors, aut_elem);
+	g_list_free (aut_elem);
+    }
 
     g_list_foreach (tokens, text_free, NULL);
     g_list_free (tokens);
