@@ -22,7 +22,7 @@
 */
 
 #ifdef HAVE_CONFIG_H
-#include "pyblioconf.h"
+#include "config.h"
 #endif
 
 #include <errno.h>
