@@ -18,16 +18,18 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: pybliographer.py,v 1.14 1999/08/16 09:38:09 gobry Exp $
+# $Id: pybliographer.py,v 1.15 1999/08/25 11:07:27 gobry Exp $
 
 version  = "@version@"
 progname = "@package@"
 
 lib_pybdir  = "@libpyb@"
 data_pybdir = "@datapyb@"
+localedir   = "@localedir@"
+
 
 def print_version ():
-	print 'This is ' + progname + ", version " + version
+	print "This is %s, version %s" % (progname, version)
 
 def copyright ():
 	print """Copyright (C) 1998-1999 Frederic GOBRY
@@ -58,6 +60,7 @@ import sys
 import os
 import getopt, string
 
+
 sys.ps1 = 'python > '
 
 sys.path.append (data_pybdir)
@@ -118,18 +121,15 @@ for opt, value in optlist:
 if not quiet:
 	print_version ()
 	copyright ()
-	print """
-Useful commands:
-	help     to get some help
-	quit     to quit
-"""
+	print "Useful commands:\n	help     to get some help\n	quit     to quit\n"
 
 # ---------- Lire les fichiers de conf
 
 user_global = {'__builtins__' : __builtins__,
-	       '__name__'   : progname,
-	       'pyb_prefix' : data_pybdir,
-	       'quiet' : quiet,
+	       '__name__'     : progname,
+	       'pyb_prefix'   : data_pybdir,
+	       'quiet'        : quiet,
+	       'version'      : version
 	       }
 
 user_local  = {}
